using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;



namespace XNACS1Lib
{
    /// <summary>
    /// This is the base class for your program. To work with the XNACS1Lib, your main class must subclass
    /// from this class.
    /// </summary>
    /// <remark>
    /// This is the base class for your program. To work with the XNACS1Lib, your main class must subclass
    /// from this class.
    /// </remark>
    internal class XNACS1Internal : Microsoft.Xna.Framework.Game
    {

        #region Private implementation subclasses have no need to have knowledge or to access any of these

        private const float DefaultWorldMinX = 0.0f;
        private const float DefaultWorldMinY = 0.0f;
        private const float DefualtWorldWidth = 100.0f;
        internal Vector2 RequestedWorldMin;
        internal float RequestedWorldWidth;

        internal XNACS1Base m_TheApp;
        static internal XNACS1Internal sm_GraphicsAccess = null;

        // lower res HDTV: 1280x720 same aspect ratio
        private int MAX_WIDTH = 1920;       // HDTV resolution
        private int MAX_HEIGHT = 1080;
        private float UseAspectRatio;

        private int AppWindowWidth;
        private int AppWindowHeight;

        private const double UPDATE_PERIOD = 25;       // milli seconds
        private bool NeedRedraw = false;

        private const float DrawViewTop = 0.0f;
        private const float DrawViewLeft = 0.0f;
        private float DrawViewWidth;
        private float DrawViewHeight;

        internal XNACS1Rectangle m_WorldBound;

        internal GraphicsDeviceManager m_GraphicsManager;
        internal GraphicsDeviceManager GraphicsManager { get { return m_GraphicsManager; } }

        internal bool m_ZuneInLandscape = false;

        internal String m_TopStatus;
        internal String m_BottomStatus;

        internal double m_LastTimeStamp;

        // Font support
        internal XNACS1LibFontManager m_FontManager;

        // Audio Support
        internal SoundManager mAudioManager;

        // Drawing support
        internal XNACS1LibDrawHelper m_DrawHelper = null;
        #endregion  private data of XNACS1Lib class


        internal XNACS1Internal(XNACS1Base theApp)
        {
#if ZUNE
            if (m_ZuneInLandscape) {
                MAX_WIDTH = 320;       
                MAX_HEIGHT = 240;
            } else {
                MAX_WIDTH = 240;       
                MAX_HEIGHT = 320;
            }
            
#endif
            UseAspectRatio = (float)MAX_WIDTH / (float)MAX_HEIGHT; 

            m_GraphicsManager = new GraphicsDeviceManager(this);

            RequestedWorldMin = new Vector2(DefaultWorldMinX, DefaultWorldMinY);
            RequestedWorldWidth = DefualtWorldWidth;

            m_WorldBound = new XNACS1Rectangle(RequestedWorldMin, RequestedWorldWidth, RequestedWorldWidth);
            m_WorldBound.Color = Color.CornflowerBlue;

            mAudioManager = new SoundManager(Content);

            m_TopStatus = null;
            m_BottomStatus = null;

            m_LastTimeStamp = 0.0;

            sm_GraphicsAccess = this;

            m_TheApp = theApp;

            NeedRedraw = true;
        }

        protected override void LoadContent()
        {
            base.LoadContent();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            AppWindowWidth = m_GraphicsManager.GraphicsDevice.DisplayMode.Width;
            AppWindowHeight = m_GraphicsManager.GraphicsDevice.DisplayMode.Height;
#if ZUNE
            AppWindowWidth = MAX_WIDTH;
            AppWindowHeight = MAX_HEIGHT;

            AppWindowHeight = (int)((((float)AppWindowWidth) / UseAspectRatio) + 0.5f);
            DrawViewHeight = AppWindowHeight;
            DrawViewWidth = AppWindowWidth;

#else
    #if XBOX360
            if (AppWindowWidth > MAX_WIDTH)
                AppWindowWidth = MAX_WIDTH;
    #else
            // PC
            AppWindowWidth = (int)(AppWindowWidth * 0.95f);
            if (AppWindowWidth > MAX_WIDTH)
                AppWindowWidth = MAX_WIDTH;
    #endif

            AppWindowHeight = (int)((((float)AppWindowWidth) / UseAspectRatio) + 0.5f);
            DrawViewHeight = AppWindowHeight;
            DrawViewWidth = AppWindowWidth;

            m_GraphicsManager.PreferredBackBufferWidth = AppWindowWidth;
            m_GraphicsManager.PreferredBackBufferHeight = AppWindowHeight;
            m_GraphicsManager.ApplyChanges();
#endif


            m_FontManager = new XNACS1LibFontManager(m_GraphicsManager.GraphicsDevice, Content, AppWindowHeight);
            CreateDrawHelper();

            m_TheApp.InternalInit();
        }

        internal void CreateDrawHelper()
        {
            // World to NDC transform
            float height;
            height = (DrawViewHeight / DrawViewWidth) * RequestedWorldWidth;

            Vector2 min, max;
            min = new Vector2(RequestedWorldMin.X, RequestedWorldMin.Y);
            max = new Vector2((min.X + RequestedWorldWidth), (min.Y + height));
            Vector2 size = max - min;
            Vector2 center = min + (size * 0.5f);
            m_WorldBound.Center = center;
            m_WorldBound.Width = size.X;
            m_WorldBound.Height = size.Y;
            if (null == m_DrawHelper)
            {
                m_DrawHelper = new XNACS1LibDrawHelper(m_GraphicsManager.GraphicsDevice, Content, m_WorldBound, m_FontManager, new Vector2(AppWindowWidth, AppWindowHeight));
                XNACS1Primitive.SetDrawHelper(m_DrawHelper);
            }
            else
            {
                // update the transformation matrices
                m_DrawHelper.SetTransformationMatrices(m_WorldBound, new Vector2(AppWindowWidth, AppWindowHeight));
            }
            m_DrawHelper.SetZuneLandscape(m_ZuneInLandscape);
        }

        /// <summary>
        /// This is called by XNA, we are not going to cause drawing here, instead we will draw
        /// periodically at 40 times per second (in LocalDraw()).
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);
            if (NeedRedraw)
                LocalDraw();
            NeedRedraw = false;
        }

        /// <summary>
        /// This is XNA's default update routine, we will count time and when 25 m-second is up,
        /// we will call LocalDraw(). And we expect the user to inherite UpdateAndDraw().
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            if ((gameTime.TotalRealTime.TotalMilliseconds - m_LastTimeStamp) > UPDATE_PERIOD)
            {
                m_LastTimeStamp = gameTime.TotalRealTime.TotalMilliseconds;
                XNACS1LibInputManager.UpdateGamePadState();
                NeedRedraw = true;

                if (!XNACS1Base.World.Paused)
                    m_DrawHelper.TravelAll();
                
                m_TheApp.InternalUpdate();
            }
        }

    
        private void LocalDraw()
        {
            m_DrawHelper.BeginDraw(SpriteBlendMode.AlphaBlend);

                GraphicsManager.GraphicsDevice.Clear(m_WorldBound.Color);

                m_WorldBound.Draw();

                m_DrawHelper.DrawAll();
                m_TheApp.InternalDraw();

                m_FontManager.DrawFonts(m_TopStatus, m_BottomStatus);

            m_DrawHelper.EndDraw();

        }
    }
}